@@ -21,9 +21,9 @@
 #include "apr_arch_file_io.h"
 #endif /* APR_HAS_SENDFILE */
 
-/* sys/sysctl.h is only needed on FreeBSD for include_hdrs_in_length() */
-#if defined(__FreeBSD__) && defined(HAVE_SYS_SYSCTL_H)
-#include <sys/sysctl.h>
+/* osreldate.h is only needed on FreeBSD for sendfile detection */
+#if defined(__FreeBSD__)
+#include <osreldate.h>
 #endif
 
 apr_status_t apr_socket_send(apr_socket_t *sock, const char *buf, 
@@ -316,8 +316,7 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
                       *len);   /* number of bytes to send */
     } while (rv == -1 && errno == EINTR);
 
-    if ((rv == -1) && 
-(errno == EAGAIN || errno == EWOULDBLOCK) 
+    if ((rv == -1) && (errno == EAGAIN || errno == EWOULDBLOCK) 
                    && (sock->timeout > 0)) {
 do_select:
         arv = apr_wait_for_io_or_timeout(NULL, sock, 0);
@@ -388,56 +387,7 @@ do_select:
     return rv < 0 ? errno : APR_SUCCESS;
 }
 
-#elif defined(__FreeBSD__)
-
-static int include_hdrs_in_length(void)
-{
-#ifdef HAVE_SYS_SYSCTL_H
-/* this assumes: 
- *   if the header exists, so does the sysctlbyname() syscall, and 
- *   if the header doesn't exist, the kernel is really old
- */
-
-/* see 
- * http://www.freebsd.org/cgi/cvsweb.cgi/src/sys/sys/param.h#rev1.61.2.29 
- * for kernel version number
- * 
- * http://www.freebsd.org/cgi/cvsweb.cgi/src/sys/kern/uipc_syscalls.c#rev1.65.2.10
- * for the sendfile patch
- */
-#define KERNEL_WITH_SENDFILE_LENGTH_FIX 460001
-
-    typedef enum { UNKNOWN = 0, 
-                   NEW, 
-                   OLD 
-                 } api_e;
-
-    static api_e api; 
-    int kernel_version;  
-    apr_size_t kernel_version_size;
-
-    if (api != UNKNOWN) {
-        return (api == OLD);
-    }
-    kernel_version = 0;    /* silence compiler warning */
-    kernel_version_size = sizeof(kernel_version);
-    if (sysctlbyname("kern.osreldate", &kernel_version, 
-                     &kernel_version_size, NULL, NULL) == 0 &&
-        kernel_version < KERNEL_WITH_SENDFILE_LENGTH_FIX) {
-        api = OLD;
-        return 1;
-    }
-    /* size of kern.osreldate's output might change in the future 
-     * causing the sysctlbyname to fail,
-     * but if it's the future, we should use the newer API 
-     */
-    api = NEW;
-    return 0;
-#else
-    /* the build system's kernel is older than 3.4.  Use the old API */
-    return 1;
-#endif
-}
+#elif defined(__FreeBSD__) || defined(__DragonFly__)
 
 /* Release 3.1 or greater */
 apr_status_t apr_socket_sendfile(apr_socket_t * sock, apr_file_t * file,
@@ -456,7 +406,8 @@ apr_status_t apr_socket_sendfile(apr_socket_t * sock, apr_file_t * file,
         hdtr = &no_hdtr;
     }
 
-    else if (hdtr->numheaders && include_hdrs_in_length()) {
+#if defined(__FreeBSD_version) && __FreeBSD_version < 460001
+    else if (hdtr->numheaders) {
 
         /* On early versions of FreeBSD sendfile, the number of bytes to send 
          * must include the length of the headers.  Don't look at the man page 
@@ -469,6 +420,7 @@ apr_status_t apr_socket_sendfile(apr_socket_t * sock, apr_file_t * file,
             bytes_to_send += hdtr->headers[i].iov_len;
         }
     }
+#endif
 
     headerstruct.headers = hdtr->headers;
     headerstruct.hdr_cnt = hdtr->numheaders;
@@ -1001,6 +953,7 @@ apr_status_t apr_socket_sendfile(apr_socket_t *sock, apr_file_t *file,
 #error APR has detected sendfile on your system, but nobody has written a
 #error version of it for APR yet.  To get past this, either write 
 #error apr_socket_sendfile or change APR_HAS_SENDFILE in apr.h to 0.
-#endif /* __linux__, __FreeBSD__, __HPUX__, _AIX, __MVS__, Tru64/OSF1 */
+#endif /* __linux__, __FreeBSD__, __DragonFly__, __HPUX__, _AIX, __MVS__,
+	  Tru64/OSF1 */
 
 #endif /* APR_HAS_SENDFILE */
